/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.sensor.project;

import java.awt.Image;
import java.awt.datatransfer.Transferable;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import javax.swing.Action;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.sensor.ChangePaletteAction;
import nl.cloudfarming.client.sensor.SensorDataObject;
import nl.cloudfarming.client.sensor.SensorFileLayer;
import org.netbeans.spi.project.ui.LogicalViewProvider;
import org.netbeans.spi.project.ui.support.CommonProjectActions;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.nodes.NodeTransfer;
import org.openide.util.Exceptions;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;

/**
 *
 * @author Timon Veenstra
 */
public class SensorProjectLogicalView implements LogicalViewProvider {

    private final SensorProject project;

    public SensorProjectLogicalView(SensorProject project) {
        this.project = project;
    }

    @Override
    public Node createLogicalView() {
        try {
            //Get the Text directory, creating if deleted
            FileObject text = project.getSensorFolder(true);

            //Get the DataObject that represents it
            DataFolder textDataObject = DataFolder.findFolder(text);

            //Get its default node-we'll wrap our node around it to change the
            //display name, icon, etc
            Node realTextFolderNode = textDataObject.getNodeDelegate();

            //This FilterNode will be our project node
            return new SensorNode(realTextFolderNode, project);

        } catch (DataObjectNotFoundException donfe) {
            Exceptions.printStackTrace(donfe);
            //Fallback-the directory couldn't be created -
            //read-only filesystem or something evil happened
            return new AbstractNode(Children.LEAF);
        }
    }

    @Override
    public Node findPath(Node root, Object target) {
        return null;
    }

    /** This is the node you actually see in the project tab for the project */
    private final class SensorNode extends FilterNode {

        final SensorProject project;

        public SensorNode(Node node, SensorProject project) throws DataObjectNotFoundException {
            super(node, new SensorNodeChildren(node),
                    //,NodeFactorySupport.createCompositeChildren( "Projects/nl-cloudfarming-client-sensor/Nodes"),
                    //The projects system wants the project in the Node's lookup.
                    //NewAction and friends want the original Node's lookup.
                    //Make a merge of both
                    new ProxyLookup(new Lookup[]{Lookups.singleton(project),
                        node.getLookup()
                    }));
            this.project = project;
        }

        @Override
        public Action[] getActions(boolean arg0) {
            return new Action[]{
                        //CommonProjectActions.newFileAction(),
                        CommonProjectActions.copyProjectAction(),
                        CommonProjectActions.deleteProjectAction(),
                        CommonProjectActions.setAsMainProjectAction(),
                        CommonProjectActions.closeProjectAction(),
                        new ImportSensorFiles(project)
                    };
        }

        @Override
        public Image getIcon(int type) {
            return ImageUtilities.loadImage("nl/cloudfarming/client/sensor/project/sensor.png");
        }

        @Override
        public Image getOpenedIcon(int type) {
            return getIcon(type);
        }

        @Override
        public String getDisplayName() {
            return project.getProjectDirectory().getName();
        }
        
        @Override
        public PasteType getDropType(Transferable t, int action, int index) {
            return null;
        }
    }

    private final class SensorNodeChildren extends FilterNode.Children {

        public SensorNodeChildren(Node or) {
            super(or);
        }

        @Override
        protected Node[] createNodes(Node key) {
            List<Node> result = new ArrayList<>();


            for (Node node : super.createNodes(key)) {
                try {
                    FileObject sensorFileObject = node.getLookup().lookup(FileObject.class);
                    DataObject dataObject = DataObject.find(sensorFileObject);
                    if (dataObject instanceof SensorDataObject) {
                        SensorFileLayer layer = new SensorFileLayer((SensorDataObject) dataObject);
                        SensorFileNode sensorFileNode = new SensorFileNode(node, (SensorDataObject) dataObject, layer);
                        result.add(sensorFileNode);
                    }
                } catch (DataObjectNotFoundException ex) {
                    Exceptions.printStackTrace(ex);
                }
            }
            return result.toArray(new Node[0]);
        }
    }

    private class SensorFileNode extends FilterNode {

        private Node original;

        public SensorFileNode(Node original, SensorDataObject sensorDataObject, Layer layer) throws DataObjectNotFoundException {
            super(original, FilterNode.Children.LEAF, new ProxyLookup(new Lookup[]{original.getLookup(), Lookups.fixed(layer, sensorDataObject.getSensorGrid())}));
            this.original = original;
        }

        @Override
        public Action[] getActions(boolean arg0) {
            Action[] addedActions = {new ChangePaletteAction()};

            Action[] originalActions = original.getActions(arg0);
            originalActions[0].setEnabled(true);
            Action[] actions = Arrays.copyOf(originalActions, originalActions.length + addedActions.length);

            int counter = originalActions.length;
            for (Action addedAction : addedActions) {
                actions[counter] = addedAction;
            }
            return actions;
        }

        @Override
        public Image getIcon(int type) {
            return original.getIcon(type);
        }

        @Override
        public Image getOpenedIcon(int type) {
            return getIcon(type);
        }

        @Override
        public String getDisplayName() {
            return original.getName();
        }

        /**
         * This implementation only calls clipboardCopy supposing that
         * copy to clipboard and copy by d'n'd are similar.
         *
         * @return transferable to represent this node during a drag
         * @exception IOException when the
         *    cut cannot be performed
         */
        @Override
        public Transferable drag() throws IOException {
            return clipboardCopy();
        }

        /** Copy this node to the clipboard.
         *
         * @return {@link org.openide.util.datatransfer.ExTransferable.Single} with one copy flavor
         * @throws IOException if it could not copy
         * @see NodeTransfer
         */
        @Override
        public Transferable clipboardCopy() throws IOException {
            return NodeTransfer.transferable(this, NodeTransfer.CLIPBOARD_COPY);
        }
        
        @Override
        public PasteType getDropType(Transferable t, int action, int index) {
            return null;
        }
    }
}
